import 'package:flutter/material.dart';
import 'package:vone_ipfs/bean/upload_result_entity.dart';
import 'package:vone_ipfs/net/http_api.dart';
import 'package:vone_ipfs/pages/image_picker_dialog.dart';
import 'package:vone_ipfs/utils/ipfs_utils.dart';

class FileUploadView extends StatefulWidget {
  List<UploadResultEntity> selectedFile;
  Function(List<UploadResultEntity> files) filesCallback;
  Widget? header;
  bool? isAllowCloud;

  FileUploadView(this.selectedFile, this.filesCallback, {Key? key, this.header,this.isAllowCloud})
      : super(key: key);

  @override
  _FileUploadViewState createState() => _FileUploadViewState();
}

class _FileUploadViewState extends State<FileUploadView> {
  List<UploadResultEntity> files = [];

  @override
  void initState() {
    super.initState();
    setState(() {
      files = widget.selectedFile;
    });
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: ListView(
        shrinkWrap: true,
        children: _buildFileList(),
      ),
    );
  }

  _buildFileList() {
    List<Widget> filesWidget = [];
    filesWidget.add(InkWell(
      child: widget.header == null ? _buildHeader() : widget.header,
      onTap: () {
        if(widget.isAllowCloud??false){
          _fileUpload();
        }else{
          _showDialog();
        }

      },
    ));
    for (int i = 0; i < files.length; i++) {
      filesWidget.add(_buildFilesItem(i));
    }
    return filesWidget;
  }

  _buildHeader() {
    return IntrinsicHeight(
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Text(
            '上传附件',
            style: TextStyle(fontSize: 14, color: Colors.black),
          ),
          Text('上传', style: TextStyle(fontSize: 14, color: Colors.blue))

        ],
      ),
    );
  }

  _buildFilesItem(int index) {
    return Padding(
      padding: EdgeInsets.symmetric(vertical: 8),
      child: Row(
        children: [
          Image.asset(
            'assets/images/ic_pdf.png',
            package: 'vone_ipfs',
            width: 20,
            height: 24,
          ),
          SizedBox(
            width: 10,
          ),
          Expanded(
            child: Text(
              '${files[index].fileName}',
              overflow: TextOverflow.ellipsis,
              style: TextStyle(fontSize: 14, color: Colors.black87),
            ),
          ),
          InkWell(
            child: Icon(
              Icons.close,
              size: 20,
            ),
            onTap: () {
              files.removeAt(index);
              if (widget.filesCallback != null) {
                widget.filesCallback(files);
              }
              setState(() {});
            },
          )
        ],
      ),
    );
  }

  _showDialog() {
    showModalBottomSheet(
        backgroundColor: Colors.transparent,
        context: context, //state.context,
        builder: (BuildContext context) {
          return ImagePickerDialog(localCallback: () {
            _fileUpload();
          }, cloudCallback: () {
            print('云端');
          },);
        });
  }
  _fileUpload(){
    IPFSUtils.fileUpload(context, (value) {
      files.add(value);
      if (widget.filesCallback != null) {
        widget.filesCallback(files);
      }
      setState(() {});
    });
  }
}
