import 'package:flutter/material.dart';
import 'package:vone_ipfs/net/http_api.dart';
import 'package:vone_ipfs/utils/ipfs_utils.dart';
import 'package:vone_ipfs/widget/intrinsic_grid_view.dart';
import 'package:vone_ipfs/widget/vone_image.dart';

import 'image_picker_dialog.dart';

class ImageUploadPage extends StatefulWidget {
  ///图片的宽度
  final double imageWidth;

  ///图片高度
  final double imageHeight;

  ///已有的图片列表
  final List<String> images;

  ///图片形状
  final ImageShape shape;

  ///上传模式 默认多文件上传
  final UploadMode uploadMode;
  final ValueChanged<List<String>>? valueChangedCallback;
  final Widget? addImageWidget;

  ImageUploadPage(
      {Key? key,
      this.imageWidth = 110,
      this.imageHeight = 110,
      required this.images,
      this.uploadMode = UploadMode.MULTIPLE,
      this.valueChangedCallback,
      this.addImageWidget,
      this.shape = ImageShape.None})
      : super(key: key);

  @override
  ImageUploadPageState createState() => new ImageUploadPageState();
}

class ImageUploadPageState extends State<ImageUploadPage> {
  List<String> _imageList = [];

  @override
  Widget build(BuildContext context) {
    int size = _imageList.length + 1;
    if (_imageList.isNotEmpty && widget.uploadMode == UploadMode.SINGLE) {
      size = _imageList.length;
    }
    return Container(
      margin: const EdgeInsets.only(left: 10),
      child: IntrinsicGridView.vertical(
        children: [for (int i = 0; i < size; i++) _buildItem(i)],
        columnCount: 3,
        horizontalSpace: 5,
        verticalSpace: 5,
      ),
    );
  }

  Widget _buildItem(int index) {
    if (_imageList.isEmpty ||
        (index == _imageList.length &&
            widget.uploadMode == UploadMode.MULTIPLE)) {
      return InkWell(
        child: widget.addImageWidget ??
            Container(
              width: widget.imageWidth,
              height: widget.imageWidth,
              decoration: BoxDecoration(
                  color: Colors.white,
                  border: Border.all(color: const Color(0xFFA1A1A1))),
              child: const Icon(
                Icons.add,
                size: 50,
                color: Colors.black45,
              ),
            ),
        onTap: () {
          _showDialog();
        },
      );
    } else {
      return Container(
        width: widget.imageWidth,
        height: widget.imageHeight,
        alignment: Alignment.topLeft,
        child: Stack(
          alignment: Alignment.topRight,
          children: [
            VoneImage(
              imageWidth: double.infinity,
              imageHeight: double.infinity,
              imageUrl: _imageList[index],
              shape: widget.shape,
              boxFit: BoxFit.fill,
            ),
            Visibility(
              child: InkWell(
                child: Container(
                  width: 20,
                  height: 20,
                  decoration: BoxDecoration(
                      color: Colors.black.withOpacity(0.6),
                      borderRadius: BorderRadius.circular(14)),
                  child: const Icon(
                    Icons.close,
                    size: 20,
                    color: Colors.white,
                  ),
                ),
                onTap: () {
                  setState(() {
                    _imageList.removeAt(index);
                  });
                  if (widget.valueChangedCallback != null) {
                    widget.valueChangedCallback!(_imageList);
                  }
                },
              ),
              visible: widget.uploadMode != UploadMode.SINGLE,
            )
          ],
        ),
      );
    }
  }

  @override
  void initState() {
    super.initState();
    _imageList = widget.images;
  }

  _showDialog() {
    showModalBottomSheet(
        backgroundColor: Colors.transparent,
        context: context, //state.context,
        builder: (BuildContext context) {
          return ImagePickerDialog(localCallback: (){
            IPFSUtils.fileUploadSingle(context, (value) {
              String url = '${HttpApi.getFilePath}/${true}/${value.fileHash}';
              _imageList.add(url);
              setState(() {});
              if(widget.valueChangedCallback!=null){
                widget.valueChangedCallback!(_imageList);
              }
            });
          }, cloudCallback: (){
            print('云端');
          },);
        });
  }
}
